/**
 @fileOverview Twitter API Wrapper.
 @name twitter_api.js
 @author mogya <mogya at mogya.com>
 @license <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache License, Version 2.0 </a>
 */
/*
 * A JavaScript implementation of the Secure Hash Algorithm, SHA-1, as defined
 * in FIPS PUB 180-1
 * Version 2.1a Copyright Paul Johnston 2000 - 2002.
 * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
 * Distributed under the BSD License
 * See http://pajhome.org.uk/crypt/md5 for details.
 */

/*
 * Configurable variables. You may need to tweak these to be compatible with
 * the server-side, but the defaults work in most cases.
 */
var hexcase = 0;
/* hex output format. 0 - lowercase; 1 - uppercase        */
var b64pad = "";
/* base-64 pad character. "=" for strict RFC compliance   */
var chrsz = 8;
/* bits per input character. 8 - ASCII; 16 - Unicode      */

/*
 * These are the functions you'll usually want to call
 * They take string arguments and return either hex or base-64 encoded strings
 */
function hex_sha1(s) {
	return binb2hex(core_sha1(str2binb(s), s.length * chrsz));
}

function b64_sha1(s) {
	return binb2b64(core_sha1(str2binb(s), s.length * chrsz));
}

function str_sha1(s) {
	return binb2str(core_sha1(str2binb(s), s.length * chrsz));
}

function hex_hmac_sha1(key, data) {
	return binb2hex(core_hmac_sha1(key, data));
}

function b64_hmac_sha1(key, data) {
	return binb2b64(core_hmac_sha1(key, data));
}

function str_hmac_sha1(key, data) {
	return binb2str(core_hmac_sha1(key, data));
}

/*
 * Perform a simple self-test to see if the VM is working
 */
function sha1_vm_test() {
	return hex_sha1("abc") == "a9993e364706816aba3e25717850c26c9cd0d89d";
}

/*
 * Calculate the SHA-1 of an array of big-endian words, and a bit length
 */
function core_sha1(x, len) {
	/* append padding */
	x[len >> 5] |= 0x80 << (24 - len % 32);
	x[((len + 64 >> 9) << 4) + 15] = len;

	var w = Array(80);
	var a = 1732584193;
	var b = -271733879;
	var c = -1732584194;
	var d = 271733878;
	var e = -1009589776;

	for (var i = 0; i < x.length; i += 16) {
		var olda = a;
		var oldb = b;
		var oldc = c;
		var oldd = d;
		var olde = e;

		for (var j = 0; j < 80; j++) {
			if (j < 16)
				w[j] = x[i + j];
			else
				w[j] = rol(w[j - 3] ^ w[j - 8] ^ w[j - 14] ^ w[j - 16], 1);
			var t = safe_add(safe_add(rol(a, 5), sha1_ft(j, b, c, d)), safe_add(safe_add(e, w[j]), sha1_kt(j)));
			e = d;
			d = c;
			c = rol(b, 30);
			b = a;
			a = t;
		}

		a = safe_add(a, olda);
		b = safe_add(b, oldb);
		c = safe_add(c, oldc);
		d = safe_add(d, oldd);
		e = safe_add(e, olde);
	}
	return Array(a, b, c, d, e);

}

/*
 * Perform the appropriate triplet combination function for the current
 * iteration
 */
function sha1_ft(t, b, c, d) {
	if (t < 20)
		return (b & c) | ((~b) & d);
	if (t < 40)
		return b ^ c ^ d;
	if (t < 60)
		return (b & c) | (b & d) | (c & d);
	return b ^ c ^ d;
}

/*
 * Determine the appropriate additive constant for the current iteration
 */
function sha1_kt(t) {
	return (t < 20) ? 1518500249 : (t < 40) ? 1859775393 : (t < 60) ? -1894007588 : -899497514;
}

/*
 * Calculate the HMAC-SHA1 of a key and some data
 */
function core_hmac_sha1(key, data) {
	var bkey = str2binb(key);
	if (bkey.length > 16)
		bkey = core_sha1(bkey, key.length * chrsz);

	var ipad = Array(16), opad = Array(16);
	for (var i = 0; i < 16; i++) {
		ipad[i] = bkey[i] ^ 0x36363636;
		opad[i] = bkey[i] ^ 0x5C5C5C5C;
	}

	var hash = core_sha1(ipad.concat(str2binb(data)), 512 + data.length * chrsz);
	return core_sha1(opad.concat(hash), 512 + 160);
}

/*
 * Add integers, wrapping at 2^32. This uses 16-bit operations internally
 * to work around bugs in some JS interpreters.
 */
function safe_add(x, y) {
	var lsw = (x & 0xFFFF) + (y & 0xFFFF);
	var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
	return (msw << 16) | (lsw & 0xFFFF);
}

/*
 * Bitwise rotate a 32-bit number to the left.
 */
function rol(num, cnt) {
	return (num << cnt) | (num >>> (32 - cnt));
}

/*
 * Convert an 8-bit or 16-bit string to an array of big-endian words
 * In 8-bit function, characters >255 have their hi-byte silently ignored.
 */
function str2binb(str) {
	var bin = Array();
	var mask = (1 << chrsz) - 1;
	for (var i = 0; i < str.length * chrsz; i += chrsz)
		bin[i >> 5] |= (str.charCodeAt(i / chrsz) & mask) << (32 - chrsz - i % 32);
	return bin;
}

/*
 * Convert an array of big-endian words to a string
 */
function binb2str(bin) {
	var str = "";
	var mask = (1 << chrsz) - 1;
	for (var i = 0; i < bin.length * 32; i += chrsz)
		str += String.fromCharCode((bin[i >> 5] >>> (32 - chrsz - i % 32)) & mask);
	return str;
}

/*
 * Convert an array of big-endian words to a hex string.
 */
function binb2hex(binarray) {
	var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
	var str = "";
	for (var i = 0; i < binarray.length * 4; i++) {
		str += hex_tab.charAt((binarray[i >> 2] >> ((3 - i % 4) * 8 + 4)) & 0xF) + hex_tab.charAt((binarray[i >> 2] >> ((3 - i % 4) * 8  )) & 0xF);
	}
	return str;
}

/*
 * Convert an array of big-endian words to a base-64 string
 */
function binb2b64(binarray) {
	var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	var str = "";
	for (var i = 0; i < binarray.length * 4; i += 3) {
		var triplet = (((binarray[i >> 2] >> 8 * (3 - i % 4)) & 0xFF) << 16) | (((binarray[i + 1 >> 2] >> 8 * (3 - (i + 1) % 4)) & 0xFF) << 8 ) | ((binarray[i + 2 >> 2] >> 8 * (3 - (i + 2) % 4)) & 0xFF);
		for (var j = 0; j < 4; j++) {
			if (i * 8 + j * 6 > binarray.length * 32)
				str += b64pad;
			else
				str += tab.charAt((triplet >> 6 * (3 - j)) & 0x3F);
		}
	}
	return str;
}

/*
* Copyright 2008 Netflix, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

/* Here's some JavaScript software for implementing OAuth.

This isn't as useful as you might hope.  OAuth is based around
allowing tools and websites to talk to each other.  However,
JavaScript running in web browsers is hampered by security
restrictions that prevent code running on one website from
accessing data stored or served on another.

Before you start hacking, make sure you understand the limitations
posed by cross-domain XMLHttpRequest.

On the bright side, some platforms use JavaScript as their
language, but enable the programmer to access other web sites.
Examples include Google Gadgets, and Microsoft Vista Sidebar.
For those platforms, this library should come in handy.
*/

// The HMAC-SHA1 signature method calls b64_hmac_sha1, defined by
// http://pajhome.org.uk/crypt/md5/sha1.js

/* An OAuth message is represented as an object like this:
 {method: "GET", action: "http://server.com/path", parameters: ...}

 The parameters may be either a map {name: value, name2: value2}
 or an Array of name-value pairs [[name, value], [name2, value2]].
 The latter representation is more powerful: it supports parameters
 in a specific sequence, or several parameters with the same name;
 for example [["a", 1], ["b", 2], ["a", 3]].

 Parameter names and values are NOT percent-encoded in an object.
 They must be encoded before transmission and decoded after reception.
 For example, this message object:
 {method: "GET", action: "http://server/path", parameters: {p: "x y"}}
 ... can be transmitted as an HTTP request that begins:
 GET /path?p=x%20y HTTP/1.0
 (This isn't a valid OAuth request, since it lacks a signature etc.)
 Note that the object "x y" is transmitted as x%20y.  To encode
 parameters, you can call OAuth.addToURL, OAuth.formEncode or
 OAuth.getAuthorization.

 This message object model harmonizes with the browser object model for
 input elements of an form, whose value property isn't percent encoded.
 The browser encodes each value before transmitting it. For example,
 see consumer.setInputs in example/consumer.js.
 */

/* This script needs to know what time it is. By default, it uses the local
 clock (new Date), which is apt to be inaccurate in browsers. To do
 better, you can load this script from a URL whose query string contains
 an oauth_timestamp parameter, whose value is a current Unix timestamp.
 For example, when generating the enclosing document using PHP:

 <script src="oauth.js?oauth_timestamp=<?=time()?>" ...

 Another option is to call OAuth.correctTimestamp with a Unix timestamp.
 */

var OAuth;
if (OAuth == null)
	OAuth = {};

OAuth.setProperties = function setProperties(into, from) {
	if (into != null && from != null) {
		for (var key in from) {
			into[key] = from[key];
		}
	}
	return into;
};

OAuth.setProperties(OAuth, // utility functions
{
	percentEncode : function percentEncode(s) {
		if (s == null) {
			return "";
		}
		if ( s instanceof Array) {
			var e = "";
			for (var i = 0; i < s.length; ++s) {
				if (e != "")
					e += '&';
				e += OAuth.percentEncode(s[i]);
			}
			return e;
		}
		s = encodeURIComponent(s);
		// Now replace the values which encodeURIComponent doesn't do
		// encodeURIComponent ignores: - _ . ! ~ * ' ( )
		// OAuth dictates the only ones you can ignore are: - _ . ~
		// Source: http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Functions:encodeURIComponent
		s = s.replace(/\!/g, "%21");
		s = s.replace(/\*/g, "%2A");
		s = s.replace(/\'/g, "%27");
		s = s.replace(/\(/g, "%28");
		s = s.replace(/\)/g, "%29");
		return s;
	},
	decodePercent : function decodePercent(s) {
		if (s != null) {
			// Handle application/x-www-form-urlencoded, which is defined by
			// http://www.w3.org/TR/html4/interact/forms.html#h-17.13.4.1
			s = s.replace(/\+/g, " ");
		}
		return decodeURIComponent(s);
	},
	/** Convert the given parameters to an Array of name-value pairs. */
	getParameterList : function getParameterList(parameters) {
		if (parameters == null) {
			return [];
		}
		if ( typeof parameters != "object") {
			return OAuth.decodeForm(parameters + "");
		}
		if ( parameters instanceof Array) {
			return parameters;
		}
		var list = [];
		for (var p in parameters) {
			list.push([p, parameters[p]]);
		}
		return list;
	},
	/** Convert the given parameters to a map from name to value. */
	getParameterMap : function getParameterMap(parameters) {
		if (parameters == null) {
			return {};
		}
		if ( typeof parameters != "object") {
			return OAuth.getParameterMap(OAuth.decodeForm(parameters + ""));
		}
		if ( parameters instanceof Array) {
			var map = {};
			for (var p = 0; p < parameters.length; ++p) {
				var key = parameters[p][0];
				if (map[key] === undefined) {// first value wins
					map[key] = '' + parameters[p][1];
				}
			}
			return map;
		}
		return parameters;
	},
	getParameter : function getParameter(parameters, name) {
		if ( parameters instanceof Array) {
			for (var p = 0; p < parameters.length; ++p) {
				if (parameters[p][0] == name) {
					return parameters[p][1];
					// first value wins
				}
			}
		} else {
			return OAuth.getParameterMap(parameters)[name];
		}
		return null;
	},
	formEncode : function formEncode(parameters) {
		var form = "";
		var list = OAuth.getParameterList(parameters);
		for (var p = 0; p < list.length; ++p) {
			var value = list[p][1];
			if (value == null)
				value = "";
			if (form != "")
				form += '&';
			form += OAuth.percentEncode(list[p][0]) + '=' + OAuth.percentEncode(value);
		}
		return form;
	},
	decodeForm : function decodeForm(form) {
		var list = [];
		var nvps = form.split('&');
		for (var n = 0; n < nvps.length; ++n) {
			var nvp = nvps[n];
			if (nvp == "") {
				continue;
			}
			var equals = nvp.indexOf('=');
			var name;
			var value;
			if (equals < 0) {
				name = OAuth.decodePercent(nvp);
				value = null;
			} else {
				name = OAuth.decodePercent(nvp.substring(0, equals));
				value = OAuth.decodePercent(nvp.substring(equals + 1));
			}
			list.push([name, value]);
		}
		return list;
	},
	setParameter : function setParameter(message, name, value) {
		var parameters = message.parameters;
		if ( parameters instanceof Array) {
			for (var p = 0; p < parameters.length; ++p) {
				if (parameters[p][0] == name) {
					if (value === undefined) {
						parameters.splice(p, 1);
					} else {
						parameters[p][1] = value;
						value = undefined;
					}
				}
			}
			if (value !== undefined) {
				parameters.push([name, value]);
			}
		} else {
			parameters = OAuth.getParameterMap(parameters);
			parameters[name] = value;
			message.parameters = parameters;
		}
	},
	setParameters : function setParameters(message, parameters) {
		var list = OAuth.getParameterList(parameters);
		for (var i = 0; i < list.length; ++i) {
			OAuth.setParameter(message, list[i][0], list[i][1]);
		}
	},
	/** Fill in parameters to help construct a request message.
	 This function doesn't fill in every parameter.
	 The accessor object should be like:
	 {consumerKey:'foo', consumerSecret:'bar', accessorSecret:'nurn', token:'krelm', tokenSecret:'blah'}
	 The accessorSecret property is optional.
	 */
	completeRequest : function completeRequest(message, accessor) {
		if (message.method == null) {
			message.method = "GET";
		}
		var map = OAuth.getParameterMap(message.parameters);
		if (map.oauth_consumer_key == null) {
			OAuth.setParameter(message, "oauth_consumer_key", accessor.consumerKey || "");
		}
		if (map.oauth_token == null && accessor.token != null) {
			OAuth.setParameter(message, "oauth_token", accessor.token);
		}
		if (map.oauth_version == null) {
			OAuth.setParameter(message, "oauth_version", "1.0");
		}
		if (map.oauth_timestamp == null) {
			OAuth.setParameter(message, "oauth_timestamp", OAuth.timestamp());
		}
		if (map.oauth_nonce == null) {
			OAuth.setParameter(message, "oauth_nonce", OAuth.nonce(6));
		}
		OAuth.SignatureMethod.sign(message, accessor);
	},
	setTimestampAndNonce : function setTimestampAndNonce(message) {
		OAuth.setParameter(message, "oauth_timestamp", OAuth.timestamp());
		OAuth.setParameter(message, "oauth_nonce", OAuth.nonce(6));
	},
	addToURL : function addToURL(url, parameters) {
		newURL = url;
		if (parameters != null) {
			var toAdd = OAuth.formEncode(parameters);
			if (toAdd.length > 0) {
				var q = url.indexOf('?');
				if (q < 0)
					newURL += '?';
				else
					newURL += '&';
				newURL += toAdd;
			}
		}
		return newURL;
	},
	/** Construct the value of the Authorization header for an HTTP request. */
	getAuthorizationHeader : function getAuthorizationHeader(realm, parameters) {
		var header = 'OAuth realm="' + OAuth.percentEncode(realm) + '"';
		var list = OAuth.getParameterList(parameters);
		for (var p = 0; p < list.length; ++p) {
			var parameter = list[p];
			var name = parameter[0];
			if (name.indexOf("oauth_") == 0) {
				header += ',' + OAuth.percentEncode(name) + '="' + OAuth.percentEncode(parameter[1]) + '"';
			}
		}
		return header;
	},
	/** Correct the time using a parameter from the URL from which the last script was loaded. */
	correctTimestampFromSrc : function correctTimestampFromSrc(parameterName) {
		parameterName = parameterName || "oauth_timestamp";
		var scripts = document.getElementsByTagName('script');
		if (scripts == null || !scripts.length)
			return;
		var src = scripts[scripts.length - 1].src;
		if (!src)
			return;
		var q = src.indexOf("?");
		if (q < 0)
			return;
		parameters = OAuth.getParameterMap(OAuth.decodeForm(src.substring(q + 1)));
		var t = parameters[parameterName];
		if (t == null)
			return;
		OAuth.correctTimestamp(t);
	},
	/** Generate timestamps starting with the given value. */
	correctTimestamp : function correctTimestamp(timestamp) {
		OAuth.timeCorrectionMsec = (timestamp * 1000) - (new Date()).getTime();
	},
	/** The difference between the correct time and my clock. */
	timeCorrectionMsec : 0,
	timestamp : function timestamp() {
		var t = (new Date()).getTime() + OAuth.timeCorrectionMsec;
		return Math.floor(t / 1000).toFixed(0);
	},
	nonce : function nonce(length) {
		var chars = OAuth.nonce.CHARS;
		var result = "";
		for (var i = 0; i < length; ++i) {
			var rnum = Math.floor(Math.random() * chars.length);
			result += chars.substring(rnum, rnum + 1);
		}
		return result;
	}
});

OAuth.nonce.CHARS = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";

/** Define a constructor function,
 without causing trouble to anyone who was using it as a namespace.
 That is, if parent[name] already existed and had properties,
 copy those properties into the new constructor.
 */
OAuth.declareClass = function declareClass(parent, name, newConstructor) {
	var previous = parent[name];
	parent[name] = newConstructor;
	if (newConstructor != null && previous != null) {
		for (var key in previous) {
			if (key != "prototype") {
				newConstructor[key] = previous[key];
			}
		}
	}
	return newConstructor;
};
/** An abstract algorithm for signing messages. */
OAuth.declareClass(OAuth, "SignatureMethod", function OAuthSignatureMethod() {
});

OAuth.setProperties(OAuth.SignatureMethod.prototype, // instance members
{
	/** Add a signature to the message. */
	sign : function sign(message) {
		var baseString = OAuth.SignatureMethod.getBaseString(message);
		var signature = this.getSignature(baseString);
		OAuth.setParameter(message, "oauth_signature", signature);
		return signature;
		// just in case someone's interested
	},
	/** Set the key string for signing. */
	initialize : function initialize(name, accessor) {
		var consumerSecret;
		if (accessor.accessorSecret != null && name.length > 9 && name.substring(name.length - 9) == "-Accessor") {
			consumerSecret = accessor.accessorSecret;
		} else {
			consumerSecret = accessor.consumerSecret;
		}
		this.key = OAuth.percentEncode(consumerSecret) + "&" + OAuth.percentEncode(accessor.tokenSecret);
	}
});

/* SignatureMethod expects an accessor object to be like this:
{tokenSecret: "lakjsdflkj...", consumerSecret: "QOUEWRI..", accessorSecret: "xcmvzc..."}
The accessorSecret property is optional.
*/
// Class members:
OAuth.setProperties(OAuth.SignatureMethod, // class members
{
	sign : function sign(message, accessor) {
		var name = OAuth.getParameterMap(message.parameters).oauth_signature_method;
		if (name == null || name == "") {
			name = "HMAC-SHA1";
			OAuth.setParameter(message, "oauth_signature_method", name);
		}
		OAuth.SignatureMethod.newMethod(name, accessor).sign(message);
	},
	/** Instantiate a SignatureMethod for the given method name. */
	newMethod : function newMethod(name, accessor) {
		var impl = OAuth.SignatureMethod.REGISTERED[name];
		if (impl != null) {
			var method = new impl();
			method.initialize(name, accessor);
			return method;
		}
		var err = new Error("signature_method_rejected");
		var acceptable = "";
		for (var r in OAuth.SignatureMethod.REGISTERED) {
			if (acceptable != "")
				acceptable += '&';
			acceptable += OAuth.percentEncode(r);
		}
		err.oauth_acceptable_signature_methods = acceptable;
		throw err;
	},
	/** A map from signature method name to constructor. */
	REGISTERED : {},
	/** Subsequently, the given constructor will be used for the named methods.
	 The constructor will be called with no parameters.
	 The resulting object should usually implement getSignature(baseString).
	 You can easily define such a constructor by calling makeSubclass, below.
	 */
	registerMethodClass : function registerMethodClass(names, classConstructor) {
		for (var n = 0; n < names.length; ++n) {
			OAuth.SignatureMethod.REGISTERED[names[n]] = classConstructor;
		}
	},
	/** Create a subclass of OAuth.SignatureMethod, with the given getSignature function. */
	makeSubclass : function makeSubclass(getSignatureFunction) {
		var superClass = OAuth.SignatureMethod;
		var subClass = function() {
			superClass.call(this);
		};
		subClass.prototype = new superClass();
		// Delete instance variables from prototype:
		// delete subclass.prototype... There aren't any.
		subClass.prototype.getSignature = getSignatureFunction;
		subClass.prototype.constructor = subClass;
		return subClass;
	},
	getBaseString : function getBaseString(message) {
		var URL = message.action;
		var q = URL.indexOf('?');
		var parameters;
		if (q < 0) {
			parameters = message.parameters;
		} else {
			// Combine the URL query string with the other parameters:
			parameters = OAuth.decodeForm(URL.substring(q + 1));
			var toAdd = OAuth.getParameterList(message.parameters);
			for (var a = 0; a < toAdd.length; ++a) {
				parameters.push(toAdd[a]);
			}
		}
		return OAuth.percentEncode(message.method.toUpperCase()) + '&' + OAuth.percentEncode(OAuth.SignatureMethod.normalizeUrl(URL)) + '&' + OAuth.percentEncode(OAuth.SignatureMethod.normalizeParameters(parameters));
	},
	normalizeUrl : function normalizeUrl(url) {
		var uri = OAuth.SignatureMethod.parseUri(url);
		var scheme = uri.protocol.toLowerCase();
		var authority = uri.authority.toLowerCase();
		var dropPort = (scheme == "http" && uri.port == 80) || (scheme == "https" && uri.port == 443);
		if (dropPort) {
			// find the last : in the authority
			var index = authority.lastIndexOf(":");
			if (index >= 0) {
				authority = authority.substring(0, index);
			}
		}
		var path = uri.path;
		if (!path) {
			path = "/";
			// conforms to RFC 2616 section 3.2.2
		}
		// we know that there is no query and no fragment here.
		return scheme + "://" + authority + path;
	},
	parseUri : function parseUri(str) {
		/* This function was adapted from parseUri 1.2.1
		 http://stevenlevithan.com/demo/parseuri/js/assets/parseuri.js
		 */
		var o = {
			key : ["source", "protocol", "authority", "userInfo", "user", "password", "host", "port", "relative", "path", "directory", "file", "query", "anchor"],
			parser : {
				strict : /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@\/]*):?([^:@\/]*))?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/
			}
		};
		var m = o.parser.strict.exec(str);
		var uri = {};
		var i = 14;
		while (i--)
		uri[o.key[i]] = m[i] || "";
		return uri;
	},
	normalizeParameters : function normalizeParameters(parameters) {
		if (parameters == null) {
			return "";
		}
		var list = OAuth.getParameterList(parameters);
		var sortable = [];
		for (var p = 0; p < list.length; ++p) {
			var nvp = list[p];
			if (nvp[0] != "oauth_signature") {
				sortable.push([OAuth.percentEncode(nvp[0]) + " "// because it comes before any character that can appear in a percentEncoded string.
				+ OAuth.percentEncode(nvp[1]), nvp]);
			}
		}
		sortable.sort(function(a, b) {
			if (a[0] < b[0])
				return -1;
			if (a[0] > b[0])
				return 1;
			return 0;
		});
		var sorted = [];
		for (var s = 0; s < sortable.length; ++s) {
			sorted.push(sortable[s][1]);
		}
		return OAuth.formEncode(sorted);
	}
});

OAuth.SignatureMethod.registerMethodClass(["PLAINTEXT", "PLAINTEXT-Accessor"], OAuth.SignatureMethod.makeSubclass(function getSignature(baseString) {
	return this.key;
}));

OAuth.SignatureMethod.registerMethodClass(["HMAC-SHA1", "HMAC-SHA1-Accessor"], OAuth.SignatureMethod.makeSubclass(function getSignature(baseString) {
	b64pad = '=';
	var signature = b64_hmac_sha1(this.key, baseString);
	return signature;
}));

try {
	OAuth.correctTimestampFromSrc();
} catch(e) {
}

/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Orginal : http://code.google.com/p/oauth-adapter/source/browse/trunk/oauth_adapter.js
 * Modified by mogya (mogya at mogya.com )
 */
var OAuthAdapter = function(params) {
	// will hold the consumer secret and consumer key as provided by the caller
	var consumerSecret = params.consumerSecret;
	var consumerKey = params.consumerKey;
	var signatureMethod = params.signatureMethod;

	// the pin or oauth_verifier returned by the authorization process window
	var pin = null;

	// will hold the request token and access token returned by the service
	var requestToken = null;
	var requestTokenSecret = null;
	var accessToken = null;
	var accessTokenSecret = null;

	// the accessor is used when communicating with the OAuth libraries to sign the messages
	var accessor = {
		consumerSecret : consumerSecret,
		tokenSecret : ''
	};

	// will hold UI components
	var window = null;
	var view = null;
	var webView = null;
	var receivePinCallback = null;

	this.loadAccessToken = function(pService) {
		Ti.API.debug('Loading access token for service [' + pService + '].');

		var file = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
		if (file.exists() == false) {
			return;
		}
		var contents = file.read();
		if (contents == null) {
			return;
		}
		var config;
		try {
			config = JSON.parse(contents.text);
		} catch(ex) {
			return;
		}
		if (!config) {
			return;
		}
		if (config.accessToken) {
			accessToken = config.accessToken;
		}
		if (config.accessTokenSecret) {
			accessTokenSecret = config.accessTokenSecret;
		}

		Ti.API.debug('Loading access token: done [accessToken:' + accessToken + '][accessTokenSecret:' + accessTokenSecret + '].');
	};
	this.saveAccessToken = function(pService) {
		Ti.API.debug('Saving access token [' + pService + '].');
		var file = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
		if (file == null) {
			file = Ti.Filesystem.createFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
		}
		file.write(JSON.stringify({
			accessToken : accessToken,
			accessTokenSecret : accessTokenSecret
		}));
		Ti.API.debug('Saving access token: done.');
	};

	this.clearAccessToken = function(pService) {
		var file = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
		if (file == null) {
			file = Ti.Filesystem.createFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
		}
		file.write(JSON.stringify({
			accessToken : null,
			accessTokenSecret : null
		}));
		accessToken = null;
		accessTokenSecret = null;
	};

	// will tell if the consumer is authorized
	this.isAuthorized = function() {
		return !(accessToken == null || accessTokenSecret == null);
	};

	// creates a message to send to the service
	var createMessage = function(pUrl, method) {
		var message = {
			action : pUrl,
			method : (method) ? method : 'POST',
			parameters : []
		};
		message.parameters.push(['oauth_consumer_key', consumerKey]);
		message.parameters.push(['oauth_signature_method', signatureMethod]);
		return message;
	};

	// returns the pin
	this.getPin = function() {
		return pin;
	};

	// requests a requet token with the given Url
	this.getRequestToken = function(pUrl, pCallback) {
		accessor.tokenSecret = '';

		var message = createMessage(pUrl);
		OAuth.setTimestampAndNonce(message);
		OAuth.SignatureMethod.sign(message, accessor);

		var client = Ti.Network.createHTTPClient();
		client.onload = function() {
			var responseParams = OAuth.getParameterMap(client.responseText);
			requestToken = responseParams['oauth_token'];
			requestTokenSecret = responseParams['oauth_token_secret'];
			Ti.API.debug('request token got the following response: ' + client.responseText);
			pCallback(client.responseText);
		};
		client.onerror = function(e) {
			Ti.API.debug(e);
			Ti.API.debug({
				error : '[' + client.status + '] ' + client.responseText
			});
			pCallback(null);
		};
		client.open('POST', pUrl, true);
		client.send(OAuth.getParameterMap(message.parameters));
	};

	// unloads the UI used to have the user authorize the application
	var destroyAuthorizeUI = function() {
		Ti.API.debug('destroyAuthorizeUI');
		// if the window doesn't exist, exit
		if (!window) {
			return;
		}

		// remove the UI
		try {
			webView.removeEventListener('load', authorizeUICallback);
			window.hide();
			window.close();
			window.remove(view);
			view.remove(webView);
			webView = null;
			view = null;
			window = null;
		} catch(ex) {
			Ti.API.debug('Cannot destroy the authorize UI. Ignoring.');
		}
	};

	// looks for the PIN everytime the user clicks on the WebView to authorize the APP
	// currently works with TWITTER
	var authorizeUICallback = function(e) {
		Ti.API.debug('authorizeUILoaded');

		var val = webView.evalJS('window.document.querySelector(\'kbd[aria-labelledby="code-desc"] > code\').innerHTML');
		if (val) {
			pin = val;
			if (receivePinCallback) {
				receivePinCallback();
			}
		}
	};

	// shows the authorization UI
	this.showAuthorizeUI = function(pUrl, pReceivePinCallback) {
		receivePinCallback = pReceivePinCallback;

		window = Ti.UI.createWindow({
			modal : true,
			fullscreen : true
		});
		var transform = Ti.UI.create2DMatrix().scale(0);
		view = Ti.UI.createView({
			top : 5,
			width : '99%',
			height : '600',
			border : 5,
			backgroundColor : 'white',
			borderColor : '#aaa',
			borderRadius : 20,
			borderWidth : 5,
			zIndex : -1,
			transform : transform
		});
		closeLabel = Ti.UI.createLabel({
			textAlign : 'right',
			font : {
				fontWeight : 'bold',
				fontSize : '12pt'
			},
			text : '(X)',
			top : 10,
			right : 12,
			height : 14
		});

		webView = Ti.UI.createWebView({
			url : pUrl,
			top : closeLabel.height + closeLabel.top,
			width : '97%',
			height : view.height - closeLabel.height - closeLabel.top - view.borderWidth * 4,
			autoDetect : [Ti.UI.AUTODETECT_NONE]
		});
		if (Titanium.version < '1.7.0') {
			view.width = 310;
			view.height = 450;
			webView.width = 300;
			webView.height = view.height - closeLabel.height - closeLabel.top - view.borderWidth * 4;
		}

		Ti.API.debug('Setting:[' + Ti.UI.AUTODETECT_NONE + ']');
		webView.addEventListener('load', authorizeUICallback);
		view.add(webView);

		closeLabel.addEventListener('click', function(e) {
			destroyAuthorizeUI();
		});
		view.add(closeLabel);

		window.add(view);
		window.open();

		var animation = Ti.UI.createAnimation();
		animation.transform = Ti.UI.create2DMatrix();
		animation.duration = 500;
		view.animate(animation);
	};

	this.getAccessToken = function(pUrl, pCallback) {
		accessor.tokenSecret = requestTokenSecret;

		var message = createMessage(pUrl);
		message.parameters.push(['oauth_token', requestToken]);
		message.parameters.push(['oauth_verifier', pin]);

		OAuth.setTimestampAndNonce(message);
		OAuth.SignatureMethod.sign(message, accessor);

		var parameterMap = OAuth.getParameterMap(message.parameters);
		for (var p in parameterMap)
		Ti.API.debug(p + ': ' + parameterMap[p]);

		var client = Ti.Network.createHTTPClient();
		client.onload = function() {
			var responseParams = OAuth.getParameterMap(client.responseText);
			accessToken = responseParams['oauth_token'];
			accessTokenSecret = responseParams['oauth_token_secret'];
			Ti.API.debug('*** get access token, Response: ' + client.responseText);
			destroyAuthorizeUI();
			pCallback();
		};
		client.onerror = function(e) {
			Ti.API.debug(e);
			destroyAuthorizeUI();
			pCallback();
		};
		client.open('POST', pUrl, true);
		client.send(parameterMap);
	};

	var oauthParams = "OAuth realm,oauth_version,oauth_consumer_key,oauth_nonce,oauth_signature,oauth_signature_method,oauth_timestamp,oauth_token".split(',');
	var makeAuthorizationHeaderString = function(params) {
		var str = '';
		for (var i = 0, len = oauthParams.length; i < len; i++) {
			var key = oauthParams[i];
			if (params[key] != undefined) {
				str += key + '="' + encodeURIComponent(params[key]) + '",';
			}
		}
		Ti.API.debug('authorization header string : ' + str);
		return str;
	};
	var removeOAuthParams = function(parameters) {
		var checkString = oauthParams.join(',') + ',';
		for (var p in parameters) {
			if (checkString.indexOf(p + ",") >= 0) {
				delete parameters[p];
			}
		}
	};
	var makePostURL = function(url, parameters) {
		var checkString = oauthParams.join(',') + ',';
		var query = [];
		var newParameters = [];
		for (var i = 0, len = parameters.length; i < len; i++) {
			var item = parameters[i];
			if (checkString.indexOf(item[0] + ",") < 0) {
				query.push(encodeURIComponent(item[0]) + "=" + encodeURIComponent(item[1]));
			} else {
				newParameters.push[item];
			}
		}
		parameters = newParameters;
		if (query.length) {
			query = query.join('&');
			return [url + ((url.indexOf('?') >= 0) ? '&' : '?') + query, parameters];
		} else {
			return [url, parameters];
		}
	};
	var makeGetURL = function(url, parameterMap) {
		var query = [];
		var keys = [];
		for (var p in parameterMap) {
			query.push(encodeURIComponent(p) + "=" + encodeURIComponent(parameterMap[p]));
		}
		query.sort();
		//(9.1.1.  Normalize Request Parameters)
		if (query.length) {
			query = query.join('&');
			return url + ((url.indexOf('?') >= 0) ? '&' : '?') + query;
		} else {
			return url;
		}
	};
	var send = function(params) {
		var pUrl = params.url;
		var pParameters = params.parameters || [];
		var pTitle = params.title;
		var pMethod = params.method || "POST";
		var resultByXML = params.resultByXML || false;

		Ti.API.debug('Sending a message to the service at [' + pUrl + '] with the following params: ' + JSON.stringify(pParameters));
		if (!this.isAuthorized()) {
			Ti.API.debug('The send status cannot be processed as the client doesn\'t have an access token. ');
			return;
		}

		accessor.tokenSecret = accessTokenSecret;
		var message = createMessage(pUrl, pMethod);
		message.parameters.push(['oauth_token', accessToken]);
		for (p in pParameters) {
			message.parameters.push(pParameters[p]);
		}
		OAuth.setTimestampAndNonce(message);
		OAuth.SignatureMethod.sign(message, accessor);
		var parameterMap = OAuth.getParameterMap(message.parameters);
		for (var p in parameterMap) {
			Ti.API.debug(p + ': ' + parameterMap[p]);
		}
		if (pMethod == "GET") {
			pUrl = makeGetURL(pUrl, parameterMap);
			parameterMap = null;
			Ti.API.debug('url for GET:' + pUrl);
		}
		var client = Ti.Network.createHTTPClient();
		client.onload = function() {
			Ti.API.debug('*** sendStatus, Response: [' + client.status + '] ' + client.responseText);
			if (("" + client.status).match(/^20[0-9]/)) {
				if (params.onSuccess) {
					params.onSuccess(client.responseText);
				}
			} else {
				if (params.onError) {
					params.onError({
						error : '[' + client.status + '] ' + client.responseText
					});
				}
			}
		};
		client.onerror = function(e) {
			Ti.API.debug(e);
			if (params.onError) {
				params.onError(e);
			}
		};
		client.open(pMethod, pUrl, true);
		client.send(parameterMap);
	};
	this.send = send;
};

/**
 @class TwitterApi. TwitterApi wrapper class. OAuth authorization, call API with API key...
 @param {hash} this function use hash as function parameters. see below.
 @param {hash} params.consumerSecret : Twitter API consumerSecret.
 @param {hash} params.consumerKey : Twitter API consumerKey.
 @description You can get consumerKey and consumerSecret when you registered your application at https://dev.twitter.com/apps/new
 */
var TwitterApi = function(params) {
	this.oAuthAdapter = new OAuthAdapter({
		consumerSecret : params.consumerSecret,
		consumerKey : params.consumerKey,
		signatureMethod : 'HMAC-SHA1'
	});

	/**
	 Do Twitter OAuth authorization and get the AccessToken. Show OAuth UI for user if needs.
	 @param {hash} this function use hash as function parameters. But there's no parameter at this version.
	 */
	this.authorize = function(params) {
		var oAuthAdapter = this.oAuthAdapter;
		oAuthAdapter.loadAccessToken('twitter');
		if (oAuthAdapter.isAuthorized() == false) {
			var receivePin = function() {
				oAuthAdapter.getAccessToken('https://api.twitter.com/oauth/access_token', function() {
					oAuthAdapter.saveAccessToken('twitter');
				});
			};
			oAuthAdapter.getRequestToken('https://api.twitter.com/oauth/request_token', function(token) {
				if (token) {
					oAuthAdapter.showAuthorizeUI('https://api.twitter.com/oauth/authorize?' + token, receivePin);
				}
			});
		}
	};

	/**
	 private function to call Twitter API.
	 @private
	 @description DO NOT call this function directly. use callApi or wrapper functions.
	 This function is a core part of callApi but it has some preconditions.
	 */
	this._callApi = function(params) {
		// replace :var in url to params.var
		var match = params.url.match(/:[^\/\.]+/g);
		if (match) {
			for (var i = 0; i < match.length; i++) {
				var index = match[i].replace(':', '');
				if (params[index]) {
					params.url = params.url.replace(match[i], params[index]);
					delete params[index];
				};
			};
			Ti.API.debug('replaced');
			Ti.API.debug(params);
		};
		var parameters = [];

		// remove known parameters and leave API parameters.
		for (var index in params) {
			if ('url' == index)
				continue;
			if ('method' == index)
				continue;
			if ('onSuccess' == index)
				continue;
			if ('onError' == index)
				continue;
			if ('parameters' == index) {
				for (var key in params.parameters) {
					parameters.push([key, params.parameters[key]]);
				};
				continue;
			}
			parameters.push([index, params[index]]);
		}

		// call API.
		var oAuthAdapter = this.oAuthAdapter;
		oAuthAdapter.loadAccessToken('twitter');
		var response = oAuthAdapter.send({
			url : params.url,
			parameters : parameters,
			method : params.method,
			onSuccess : function(response) {
				response = JSON.parse(response);
				if (params && params.onSuccess) {
					params.onSuccess(response);
				} else {
					Ti.API.error('no success handler');
				}
			},
			onError : function(error) {
				if (params.onError) {
					params.onError(error);
				} else {
					Ti.API.error('no error handler');
					Ti.API.error(error);
				}
			}
		});
	};
	/**
	 call Twitter API.
	 @param {hash} this function use hash as function parameters. see below.
	 @param {string} params.url : API endpoint url. You can put some parameters like ':id' in url. callApi will replace it by params key and value.
	 @param {string} params.method : 'GET' OR 'POST'
	 @param {function} params.onSuccess(response) : callback function called on success. response: twitter API responce.
	 @param {function} params.onError(e) : callback function called on error. e:twitter API responce.
	 @param {hash} params.parameters : API parameters. It's not recommended to write API paramaters in params directly. use params.parameters.
	 @description You don't need to call this function directly. Wrapper functions available on this class.
	 However this is still useful function to call undocumented APIs or new APIs.
	 <font size='small'></font>
	 @sample
	 var TwitterApi = require('lib/twitter_api').TwitterApi;
	 var twitter_api = new TwitterApi({
	 consumerKey:'YOUR_CONSUMER_KEY',
	 consumerSecret:'YOUR_CONSUMER_SECRET'
	 });
	 twitter_api.authorize();
	 var params = {
	 url : 'http://api.twitter.com/1/statuses/show/:id.json',
	 method : 'GET',
	 id : '12345678',
	 };
	 params.onSuccess = function(response){
	 // DO something with response.
	 };
	 params.onError = function(error){
	 Ti.API.error(error);
	 },
	 twitter_api.callApi(params);
	 */
	this.callApi = function(params) {
		var self = this;
		if (!params) {
			Ti.API.error('twitter api params error. no params.');
			return false;
		}
		if (!params.url) {
			Ti.API.error('twitter api params error. no params.url.');
			return false;
		};
		//queue capability. if this function is called before authorization, wait authorization.
		if (this.oAuthAdapter.isAuthorized()) {
			return this._callApi(params);
		} else {
			setTimeout(function() {
				self.callApi(params);
			}, 1000);
		}
		return false;
	};

	/**
	 * clear twitter auth infomations. Forget accesstoken and remove the file.
	 */
	this.clear_accesstoken = function() {
		this.oAuthAdapter.clearAccessToken('twitter');
	};

	/**
	 * Verify if the user is logged on Twitter.
	 */
	this.isAuthorized = function() {
		var oAuthAdapter = this.oAuthAdapter;
		oAuthAdapter.loadAccessToken('twitter');
		return oAuthAdapter.isAuthorized();
	};

	/**
	 Twitter APIs
	 We can use same parameter of Twitter API.
	 see http://dev.twitter.com/doc
	 Additional params for all APIs.
	 params.onSuccess(response)
	 callback function when API returns successfully.
	 params.onError(error)
	 callback function for illegal situation.
	 */

	/** Twitter API :  statuses.public_timeline
	 */
	this.statuses_public_timeline = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/public_timeline.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.home_timeline
	 */
	this.statuses_home_timeline = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/home_timeline.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.friends_timeline
	 */
	this.statuses_friends_timeline = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/friends_timeline.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.user_timeline
	 */
	this.statuses_user_timeline = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/user_timeline.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.mentions
	 */
	this.statuses_mentions = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/mentions.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.retweeted_by_me
	 */
	this.statuses_retweeted_by_me = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/retweeted_by_me.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.retweeted_to_me
	 */
	this.statuses_retweeted_to_me = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/retweeted_to_me.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.show
	 */
	this.statuses_show = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/show/:id.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.update
	 */
	this.statuses_update = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/update.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : statuses.destroy
	 */
	this.statuses_destroy = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/destroy/:id.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : statuses.retweet
	 */
	this.statuses_retweet = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/retweet/:id.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : statuses.retweets
	 */
	this.statuses_retweets = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/retweets/:id.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.retweeted_by
	 */
	this.statuses_retweeted_by = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/:id/retweeted_by.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.retweeted_by_ids
	 */
	this.statuses_retweeted_by_ids = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/:id/retweeted_by/:ids.json';
		params.method = 'GET';
		return this.callApi(params);
	};

	/** Twitter API : users.show
	 */
	this.users_show = function(params) {
		params.url = 'http://api.twitter.com/1/users/show.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : users.lookup
	 */
	this.users_lookup = function(params) {
		params.url = 'http://api.twitter.com/1/users/lookup.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : users.search
	 */
	this.users_search = function(params) {
		params.url = 'http://api.twitter.com/1/users/search.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : users.suggestions
	 */
	this.users_suggestions = function(params) {
		params.url = 'http://api.twitter.com/1/users/suggestions.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : users.suggestions.twitter
	 */
	this.users_suggestions_twitter = function(params) {
		params.url = 'http://api.twitter.com/1/users/suggestions/:slug.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : users.profile_image.twitter
	 */
	this.users_profile_image_twitter = function(params) {
		params.url = 'http://api.twitter.com/1/users/profile_image/:screen_name.json';
		params.method = 'GET';
		return this.callApi(params);
	};

	/** Twitter API : statuses.friends
	 */
	this.statuses_friends = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/friends.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : statuses.followers
	 */
	this.statuses_followers = function(params) {
		params.url = 'http://api.twitter.com/1/statuses/followers.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : trends
	 */
	this.trends = function(params) {
		params.url = 'http://api.twitter.com/1/trends.json';
		params.method = 'GET';
		return this.callApi(params);
	};

	/** Twitter API : trends.current
	 */
	this.trends_current = function(params) {
		params.url = 'http://api.twitter.com/1/trends/current.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : trends.daily
	 */
	this.trends_daily = function(params) {
		params.url = 'http://api.twitter.com/1/trends/daily.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : trends.weekly
	 */
	this.trends_weekly = function(params) {
		params.url = 'http://api.twitter.com/1/trends/weekly.json';
		params.method = 'GET';
		return this.callApi(params);
	};

	/** Twitter API : user/lists_create
	 Creates a new list for the authenticated user.
	 */
	this.user_lists_create = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists.json';
		params.method = 'POST';
	};
	/** Twitter API :  :user/lists/:id
	 Updates the specified list.
	 */
	this.user_lists_update = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists/:id.json';
		params.method = 'POST';
	};
	/** Twitter API :  :user/lists
	 List the lists of the specified user.
	 */
	this.user_lists_of_user = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists.json';
		params.method = 'GET';
	};
	/** Twitter API :  :user/lists/:id
	 Show the specified list.
	 */
	this.user_lists_get = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists/:id.json';
		params.method = 'GET';
	};
	/** Twitter API :  :user/lists/:id
	 Deletes the specified list
	 */
	this.user_lists_delete = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists/:id.json';
		params.method = 'POST';
		params._method = 'DELETE';
	};
	/** Twitter API :  :user/lists/:id/statuses
	 Show tweet timeline for members of the specified list.
	 */
	this.user_lists_statuses = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists/:id/statuses.json';
		params.method = 'GET';
	};
	/** Twitter API :  :user/lists/memberships
	 List the lists the specified user has been added to.
	 */
	this.user_lists_memberships = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists/memberships.json';
		params.method = 'GET';
	};
	/** Twitter API :  :user/lists/subscriptions
	 List the lists the specified user follows.
	 */
	this.user_lists_subscriptions = function(params) {
		params.url = 'http://api.twitter.com/1/:user/lists/subscriptions.json';
		params.method = 'GET';
	};

	/** Twitter API : direct_messages
	 */
	this.direct_messages = function(params) {
		params.url = 'http://api.twitter.com/1/direct_messages.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : direct_messages.sent
	 */
	this.direct_messages_sent = function(params) {
		params.url = 'http://api.twitter.com/1/direct_messages/sent.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : direct_messages.new
	 */
	this.direct_messages_new = function(params) {
		params.url = 'http://api.twitter.com/1/direct_messages/new.json';
		params.method = 'POST';
		return this.callApi(params);
	};

	/** Twitter API : friendships.create
	 */
	this.friendships_create = function(params) {
		params.url = 'http://api.twitter.com/1/friendships/create.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : friendships.destroy
	 */
	this.friendships_destroy = function(params) {
		params.url = 'http://api.twitter.com/1/friendships/destroy/:id.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : friendships.exists
	 */
	this.friendships_exists = function(params) {
		params.url = 'http://api.twitter.com/1/friendships/exists.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : friendships.show
	 */
	this.friendships_show = function(params) {
		params.url = 'http://api.twitter.com/1/friendships/show.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : friendships.incoming
	 */
	this.friendships_incoming = function(params) {
		params.url = 'http://api.twitter.com/1/friendships/incoming.json';
		params.method = 'GET';
		return this.callApi(params);
	};

	/** Twitter API : account/verify_credentials
	 */
	this.account_verify_credentials = function(params) {
		params.url = 'http://api.twitter.com/1/account/verify_credentials.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : account.rate_limit_status
	 */
	this.account_rate_limit_status = function(params) {
		params.url = 'http://api.twitter.com/1/account/rate_limit_status.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : account.end_session
	 */
	this.account_end_session = function(params) {
		params.url = 'http://api.twitter.com/1/account/end_session.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : account.update_delivery_device
	 */
	this.account_update_delivery_device = function(params) {
		params.url = 'http://api.twitter.com/1/account/update_delivery_device.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : account.update_profile_colors
	 */
	this.account_update_profile_colors = function(params) {
		params.url = 'http://api.twitter.com/1/account/update_profile_colors.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : account.update_profile_image
	 */
	this.account_update_profile_image = function(params) {
		params.url = 'http://api.twitter.com/1/account/update_profile_image.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : account.update_profile_background_image
	 */
	this.account_update_profile_background_image = function(params) {
		params.url = 'http://api.twitter.com/1/account/update_profile_background_image.json';
		params.method = 'POST';
		return this.callApi(params);
	};

	/** Twitter API : favorites.create
	 */
	this.favorites = function(params) {
		params.url = 'http://api.twitter.com/1/favorites.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : favorites.create
	 */
	this.favorites_create = function(params) {
		params.url = 'http://api.twitter.com/1/favorites/create/:id.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : favorites.destroy
	 */
	this.favorites_destroy = function(params) {
		params.url = 'http://api.twitter.com/1/favorites/destroy:id.json';
		params.method = 'POST';
		return this.callApi(params);
	};

	/** Twitter API : blocks.create
	 */
	this.blocks_create = function(params) {
		params.url = 'http://api.twitter.com/1/blocks/create.json';
		params.method = 'POST';
		return this.callApi(params);
	};

	/** Twitter API : blocks.destroy
	 */
	this.blocks_destroy = function(params) {
		params.url = 'http://api.twitter.com/1/blocks/destroy.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : blocks.exists
	 */
	this.blocks_exists = function(params) {
		params.url = 'http://api.twitter.com/1/blocks/exists.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : blocks.blocking
	 */
	this.blocks_blocking = function(params) {
		params.url = 'http://api.twitter.com/1/blocks/blocking.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : blocks.blocking_ids
	 */
	this.blocks_blocking_ids = function(params) {
		params.url = 'http://api.twitter.com/1/blocks/blocking/:ids.json';
		params.method = 'GET';
		return this.callApi(params);
	};

	/** Twitter API :  report_spam
	 */
	this.report_spam = function(params) {
		params.url = 'http://api.twitter.com/1/report_spam.json';
		params.method = 'POST';
		return this.callApi(params);
	};

	/** Twitter API : saved_searches
	 */
	this.saved_searches = function(params) {
		params.url = 'http://api.twitter.com/1/saved_searches.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : saved_searches.show
	 */
	this.saved_searches_show = function(params) {
		params.url = 'http://api.twitter.com/1/saved_searches/show.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : saved_searches.create
	 */
	this.saved_searches_create = function(params) {
		params.url = 'http://api.twitter.com/1/saved_searches/create.json';
		params.method = 'POST';
		return this.callApi(params);
	};
	/** Twitter API : saved_searches.delete
	 */
	this.saved_searches_delete = function(params) {
		params.url = 'http://api.twitter.com/1/saved_searches/destroy.json';
		params.method = 'POST';
		return this.callApi(params);
	};

	/** Twitter API : geo.search
	 */
	this.geo_search = function(params) {
		params.url = 'http://api.twitter.com/1/geo/search.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : geo.similar_places
	 */
	this.geo_similar_places = function(params) {
		params.url = 'http://api.twitter.com/1/geo/similar_places.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : geo.reverse_geocode
	 */
	this.geo_reverse_geocode = function(params) {
		params.url = 'http://api.twitter.com/1/geo/reverse_geocode.json';
		params.method = 'GET';
		return this.callApi(params);
	};
	/** Twitter API : geo.id.place
	 */
	this.geo_id_place = function(params) {
		params.url = 'http://api.twitter.com/1/geo/id/:place_id.json';
		params.method = 'GET';
		return this.callApi(params);
	};
};
exports.TwitterApi = TwitterApi; 